"use strict";
/**
 * @file This plugin is to access classification image data from different sources. Make sure that
 * the data is conform to expectation.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const pipcook_core_1 = require("@pipcook/pipcook-core");
const glob_promise_1 = __importDefault(require("glob-promise"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
class DataLoader {
    constructor(dataPairs) {
        this.dataPairs = dataPairs;
    }
    len() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.dataPairs.length;
        });
    }
    getItem(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                data: this.dataPairs[id].image,
                label: this.dataPairs[id].label
            };
        });
    }
}
/**
 * merge all possible values of labels. Get the map between label and numeric value
 * @param dataPath
 */
const getLabelMap = (dataPath) => __awaiter(void 0, void 0, void 0, function* () {
    const labelSet = new Set();
    const trainFileNames = yield glob_promise_1.default(path.join(dataPath, 'train', '*.xml'));
    for (let j = 0; j < trainFileNames.length; j++) {
        const fileName = trainFileNames[j];
        const imageData = yield pipcook_core_1.parseAnnotation(fileName);
        imageData.annotation.object.forEach((object) => {
            labelSet.add(object.name[0]);
        });
    }
    const labelArray = Array.from(labelSet);
    const labelMap = {};
    labelArray.forEach((label, index) => {
        labelMap[label] = index;
    });
    return labelMap;
});
const getValidPair = (dataPath, labelMap) => __awaiter(void 0, void 0, void 0, function* () {
    const annotationPaths = yield glob_promise_1.default(path.join(dataPath, '*.xml'));
    const pairs = [];
    for (let i = 0; i < annotationPaths.length; i++) {
        const fileName = annotationPaths[i];
        const imageData = yield pipcook_core_1.parseAnnotation(fileName);
        const imageName = imageData.annotation.filename[0];
        if (fs.existsSync(path.join(dataPath, imageName))) {
            imageData.annotation.object.forEach((object) => {
                const label = {
                    name: object.name[0],
                    categoryId: labelMap[object.name[0]]
                };
                if (object.bndbox) {
                    label.bndbox = {
                        xmin: Number(object.bndbox[0].xmin[0]),
                        ymin: Number(object.bndbox[0].ymin[0]),
                        xmax: Number(object.bndbox[0].xmax[0]),
                        ymax: Number(object.bndbox[0].ymax[0])
                    };
                }
                if (object.segmentation) {
                    label.segmentation = object.segmentation[0];
                }
                pairs.push({
                    annotation: fileName,
                    image: path.join(dataPath, imageName),
                    label
                });
            });
        }
    }
    if (pairs.length > 0) {
        yield pipcook_core_1.convertPascal2CocoFileOutput(Array.from(new Set(pairs.map((pair) => pair.annotation))), path.join(dataPath, 'annotation.json'));
    }
    return pairs;
});
/**
 * The plugin used to access data from different sources. It will detect all possible values of labels and
 * merge them into numeric expressions.
 */
const cocoDataAccess = (args) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const { dataDir } = args;
    const labelMap = yield getLabelMap(dataDir);
    const trainPair = yield getValidPair(path.join(dataDir, 'train'), labelMap);
    const validationPair = yield getValidPair(path.join(dataDir, 'validation'), labelMap);
    const testPair = yield getValidPair(path.join(dataDir, 'test'), labelMap);
    const isBitMask = (_a = trainPair[0].label.segmentation) === null || _a === void 0 ? void 0 : _a.hasOwnProperty('counts');
    const trainLoader = new DataLoader(trainPair);
    const validationLoader = new DataLoader(validationPair);
    const testLoader = new DataLoader(testPair);
    const result = {
        metadata: {
            labelMap,
            isBitMask
        },
        dataStatistics: [],
        validationResult: {
            result: true
        },
        trainAnnotationPath: path.join(dataDir, 'train', 'annotation.json'),
        validationAnnotationPath: path.join(dataDir, 'validation', 'annotation.json'),
        testAnnotationPath: path.join(dataDir, 'test', 'annotation.json')
    };
    if (trainPair.length > 0) {
        result.trainLoader = trainLoader;
    }
    if (validationPair.length > 0) {
        result.validationLoader = validationLoader;
    }
    if (testPair.length > 0) {
        result.testLoader = testLoader;
    }
    return result;
});
exports.default = cocoDataAccess;
//# sourceMappingURL=index.js.map